function points = period_classify(fhandle, xF, max_period, tolerance, varargin)

%
% Classify the given array of points according to their
% period, belonging to a certain cycle, and stability.
%
% fhandle    - function handle 
% xF         - array of points
% max_period - maximal period of points to check
% tolerance  - checking point accuracy
% varargin   - cell array of the map parameters
%
% return values:
% points - cell-array of classified periodic points. Each
%          row has three cells: the first one contains the
%          cycle coordinates, the second one is the period,
%          and the third one indicates the stability
%          ('stable', 'unstable', or 'saddle')
%


lgth = length(xF(:, 1));
points = {};


% checking if there exists an explicit Jacobi matrix function
fh = functions(fhandle);
Jname = [fh.function '_J'];
Jhandle = str2func(Jname);
Jh = functions(Jhandle);
if strcmp(Jh.file, '')   % there is no file with Jacobi matrix function
    cprintf(get_colour('warn'), 'WARNING: The function''s Jacobi matrix is not defined\n');
    cprintf(get_colour('warn'), '         and will be calculated approximately.\n\n');
    has_jacob = false;
else
    has_jacob = true;
end


num = 1;
for i = 1:lgth 
    point = xF(i, :);

    known = 0;
    if (num > 1)
        for j = 1:num-1
            cycle = points{j, 1};
            for k = 1:points{j, 2}
                if (norm(point - cycle(k, :)) < tolerance)
                    % the current point is a point of the known cycle
                    known = 1;
                    break;
                end;
            end;
        
            if (known)
                % the current point is a point of the known cycle
                break;
            end;
        end;
    end;
    
    if (known)
        % the current point is a point of the known cycle
        % do not need to count it
        continue;
    else
        % the current point defines a new cycle
        cycle = point;
        period = 0;
        for j = 1:max_period
            point = fhandle(point, varargin{:});
            if (norm(point - cycle(1, :)) < tolerance)
                period = j;
                break;
            end;
            cycle = [cycle; point];
        end
        
        if (period == 0)
            % the point has wrong period
            cprintf(get_colour('aux'), ['WARNING: The point xF(%d) is of ' ...
                    'period > %d, and will not be treated.\n'], i, max_period);
        else
            % saving new cycle
            points{num, 1} = cycle;
            points{num, 2} = period;
            
            % finding the eigenvalues
            if (has_jacob) % Jacobian is implemented
                J = Jhandle(cycle(1, :), varargin{:});
                for i = 2:period
                    J = Jhandle(cycle(i, :), varargin{:})*J;
                end;
            else % Jacobian is not implemented, find numerically
                J = jacob(fhandle, cycle(1, :), 0.0001, varargin{:});
                for i = 2:period
                    J = jacob(fhandle, cycle(i, :), 0.0001, varargin{:})*J;
                end;
            end;
            
            l = eig(J);
            idx = find(abs(l) > 1);
            if (length(idx) == 0) % stable cycle
                points{num, 3} = 'stable';
            elseif (length(idx) == 1) % saddle cycle
                points{num, 3} = 'saddle';
            else % unstable cycle
                points{num, 3} = 'unstable';
            end;
            
            num = num + 1;
        end;
    end;

end;

