function [p, arc_length] = sc_unstable_cyc(fhandle, p0, period, arc_max, d, ...
    a_min, a_max, Da_min, Da_max, D_min, cB, diverge, varargin)

%
% Find a 1D unstable manifold of a saddle point
% by using a search circle algorithm
% (see England & Krauskopf & Osinga, SIAM JADS 3(2): 161-190, 2004)
% periodic points
%
% fhandle        - function (dynamical system) handle
% p0             - saddle point
% arc_max        - desired length of the manifold arc
% d              - initial step size
%                  (if d > 0 manifold is grown forward, if d
%                  < 0 manifold is grown backward) 
% a_min, a_max   - angle bounds (a_min < ak < a_max)
% Da_min, Da_max - bounds for ak*Dk (da_min < ak*Dk < Da_max)
% D_min          - minimal step size
% cB             - bisection accuracy coefficient, should be between 0 and 1
% varargin       - cell array of the function parameters
%
% return values:
% p          - array of the computed manifold points
% arc_length - length of the computed manifold arc 
%


%diverge = 100000


load('data/default.mat', 'err'); % error for point comparison
load('data/default.mat', 'dx'); % increment for jacobian calculation

% half of the minimal step size
half_D_min = D_min/2;

p = [];
arc_length = 0;

p(1, :) = p0;

J = calc_jacob_cyc(fhandle, p0, period, dx, varargin{:})

[v, l] = eig(J);
l = diag(l);
%cprintf(get_colour('aux'), '\n');
%cprintf(get_colour('aux'), 'Eigenvalues are %.8f and %.8f\n\n', l(1), l(2));
printf('\n');
printf('Eigenvalues are %.8f and %.8f\n\n', l(1), l(2));
idx = find(abs(l) > 1);
if (length(idx) ~= 1)
    %cprintf('err', 'ERROR: The routine finds only one-dimensional manifolds.\n');
    printf('ERROR: The routine finds only one-dimensional manifolds.\n');
    return;
end;


if (l(idx) > 0)
    iters = period;
else
    iters = 2*period;
end;


p(2, :) = (p0' + d*v(:, idx))';
arc_length = abs(d);

p(3, :) = iterate(fhandle, p(2, :), iters, diverge, varargin{:});
Dk = norm(p(3, :) - p(2, :));
arc_length = arc_length + Dk;


i = 2;
k = 3;
Dk = abs(d)*(1 - 1/abs(l(idx)));

p_inv = p(2, :);


while (arc_length < arc_max)

    if (mod(k, 1000) == 0)
        %cprintf(get_colour('vital'), 'Computed upto k = %d. Arc length is %.8f.\n', k, arc_length);
        printf('Computed upto k = %d. Arc length is %.8f.\n', k, arc_length);
    end;
    
    
    % saving i for potential further computation with halved step size
    i_old = i;

    while (1)

        i = find_segment(fhandle, iters, p, i, Dk, diverge, varargin{:});
    
        if (i > k)
            % the right segment was not found
            if (Dk < D_min)
                % the step size is minimal, aborting
                %cprintf(get_colour('warn'), 'WARNING: Unable to find next point, aborting computation.\n');
                %cprintf(get_colour('final'), 'Manifold is computed upto %d point, arc length = %.8f\n', ...
                printf('WARNING: Unable to find next point, aborting computation.\n');
                printf('Manifold is computed upto %d point, arc length = %.8f\n', ...
                        k, arc_length);
                return;
            else
                % tryig to decrease step size
                Dk = Dk/2;
                if (Dk < half_D_min)
                    Dk = half_D_min;
                end;
            
                i = i_old;
                continue;
            end;
        else
            % the right segment was found
            if (i == i_old)
                p_start = p_inv;
            else
                p_start = p(i-1, :);
            end;
            p_end = p(i, :);
            
            [p_next, f_next, stat] = bisection(fhandle, iters, p(k, :), p_start, p_end, ...
                Dk, cB*Dk, diverge, varargin{:});
            
            
            if (stat == 1)
                %cprintf(get_colour('aux'), 'k = %d, bisection failed.\n', k);
                printf('k = %d, bisection failed.\n', k);
            end;
            
            ak = vector_angle(p(k, :) - p(k-1, :), f_next - p(k, :));
            if ((ak > a_max) || (Dk*ak > Da_max))
                % p_next is not acceptable
                
                if (Dk < D_min)
                    %cprintf(get_colour('warn'), 'WARNING: the step size is too small, accepting the point as is.\n');
                    printf('WARNING: the step size is too small, accepting the point as is.\n');
                    break;
                else
                    % tryig to decrease step size
                    Dk = Dk/2;
                    if (Dk < half_D_min)
                        Dk = half_D_min;
                    end;
                    
                    i = i_old;
                    continue;
                end
            else
                % p_next is acceptable
                % the step is too short: doubling
                if ((ak < a_min) && (Dk*ak < Da_min))
                    Dk = 2*Dk;
                end;
                
                break;
            end;
        end;
    end; % while (1)

    
    p_inv = p_next;
    p(k+1, :) = f_next;
    arc_length = arc_length + Dk;
    
    % check if the manifold end at a periodic point
    if (k > 100)
        f_next1 = iterate(fhandle, f_next, iters, diverge, varargin{:});
        if (norm(f_next1 - f_next) < half_D_min)
            %cprintf(get_colour('final'), '\n');
            %cprintf(get_colour('final'), 'The manifold seems to be of the finite arclength.\n');
            %cprintf(get_colour('final'), 'The end point must be a periodic point: (%.8f, %.8f).\n', f_next1(1), f_next1(2));
            %cprintf(get_colour('final'), 'Manifold is computed upto %d point, arc length = %.8f.', k, arc_length);
            printf('\n');
            printf('The manifold seems to be of the finite arclength.\n');
            printf('The end point must be a periodic point: (%.8f, %.8f).\n', f_next1(1), f_next1(2));
            printf('Manifold is computed upto %d point, arc length = %.8f.', k, arc_length);
            return;
        end;
        % check if the manifold end at a point of the doubled period
        f_next1 = iterate(fhandle, f_next1, iters, diverge, varargin{:});
        if (norm(f_next1 - f_next) < half_D_min)
            %cprintf(get_colour('final'), '\n');
            %cprintf(get_colour('final'), 'The manifold seems to be of the finite arclength.\n');
            %cprintf(get_colour('final'), 'The end point must be a periodic point: (%.8f, %.8f).\n', f_next1(1), f_next1(2));
            %cprintf(get_colour('final'), 'Manifold is computed upto %d point, arc length = %.8f.', k, arc_length);
            printf('\n');
            printf('The manifold seems to be of the finite arclength.\n');
            printf('The end point must be a periodic point: (%.8f, %.8f).\n', f_next1(1), f_next1(2));
            printf('Manifold is computed upto %d point, arc length = %.8f.', k, arc_length);
            return;
        end;
    end;
    
    k = k + 1;
    
end;

%cprintf(get_colour('final'), '\n');
%cprintf(get_colour('final'), 'Computation finished.\n');
%cprintf(get_colour('final'), 'Manifold is computed upto %d point, arc length = %.8f\n', k, arc_length);
printf('\n');
printf('Computation finished.\n');
printf('Manifold is computed upto %d point, arc length = %.8f\n', k, arc_length);







