function y = cutting_map(x, tau, zeta, h0, K2)

%
% Cutting tool map (highly interrupted map)
% [see England & Krauskopf & Osinga, SIAM JADS 3(2):
% 161-190, 2004 (example 4.4)]
%
%  x -> A11*x + A12*v
%  v -> A21*x + A22*v + F(x, v)
%
% A11 = exp(-zeta*tau)/(1 - zeta^2)*(cos(T) + zeta*sin(T))
% A12 = exp(-zeta*tau)/(1 - zeta^2)*sin(T)
% A21 = -A12
% A22 = exp(-zeta*tau)/(1 - zeta^2)*(cos(T) - zeta*sin(T))
%
% T = tau*sqrt(1 - zeta^2)
%
%            /  K2*h^(3/4),  if  h = h0 + x - A11*x - A12*v > 0
% F(x, v) = |
%            \  0,           otherwise
% 
%

if (length(x) ~= 2)
    cprintf('err', 'ERROR: First argument must be a vector with 2 elements.\n');
    y = [];
    return;
else
    root = tau*sqrt(1 - zeta^2);
    expn = exp(-zeta*tau)/(1 - zeta^2);
    A11 = expn*(cos(root) + zeta*sin(root));
    A12 = expn*sin(root);
    A21 = -A12;
    A22 = expn*(cos(root) - zeta*sin(root));
    
    h = h0 + x(1) - A11*x(1) - A12*x(2);
    if (h > 0)
        F = K2*h^(3/4);
    else
        F = 0;
    end
    
    y(1) = A11*x(1) + A12*x(2);
    y(2) = A21*x(1) + A22*x(2) + F;
end

