function Dk = find_delta(fhandle, p0, p1, Dk_est, iters, eB, diverge, varargin)

%
% Find step size Dk for   p_{k+1} = p_k + Dk*(p_k - p_{k-1})
% such that f(p_{k+1}) lies near normal vector of 
% [p_{k-1}, p_k] 
%
% fhandle  - function (dynamical system) handle
% p0       - start point of the segment (p_{k-1})
% p1       - end point of the segment (p_k)
% Dk_est   - estimate for Dk
% iters    - number of iterates needed
% eB       - bisection error
% varargin - cell array of the function parameters
%

%diverge = 100000;


% maximal number of bisection steps
bstep_max = 10;

% vector norm
p01_norm = norm(p1 - p0);

% We are searching Dk in the interval [Dk_est/2, 3/2*Dk_est]
% using bisection, thus, f(p_start) and f(p_end) should lie
% on the different sides of the normal to [p0, p1]

Dk_start = Dk_est/2;
p_start = p1 + Dk_start*(p1 - p0)/p01_norm;
f_start = iterate(fhandle, p_start, iters, diverge, varargin{:});
while (is_acute(f_start - p1, p0 - p1) ~= 1)
    % start delta is too large
    Dk_start = Dk_start/2;
    p_start = p1 + Dk_start*(p1 - p0)/p01_norm;
    f_start = iterate(fhandle, p_start, iters, diverge, varargin{:});
end % decrease start delta


Dk_end = 3/2*Dk_est;
% p_end = p1 + Dk_end*(p1 - p0)/p01_norm;
% f_end = iterate(fhandle, p_end, iters, diverge, varargin{:});
% while (is_acute(f_end - p1, p0 - p1) ~= -1)
%     % end delta is too small
%     Dk_end = 2*Dk_end;
%     p_end = p1 + Dk_end*(p1 - p0)/p01_norm;
%     f_end = iterate(fhandle, p_end, iters, diverge, varargin{:});
% end % increase end delta


bstep = 1;
while (true)
    Dk = (Dk_start + Dk_end)/2;
    p_try = p1 + Dk*(p1 - p0)/p01_norm;
    f_try = iterate(fhandle, p_try, iters, diverge, varargin{:});
    
    if (is_acute(f_try - p1, p0 - p1) == 1)
        % f(p_try) is on the same side of the normal as f(p_start)
        % moving p_start
        if ( (dot(f_try-p1, p0-p1) < eB) || (bstep > bstep_max) )
            % the point found is on the right side of the normal
            % and either is close enough to it
            % or the number of  bisection steps is too large
            break;
        end
        Dk_start = Dk;
    else
        % f(p_try) is on the same side of the normal as f(p_end)
        % moving p_end
        Dk_end = Dk;
    end        
    
    bstep = bstep + 1;
end


