function [p_next, f_next, stat] = search_circle(fhandle, iters, ...
    p_cur_0, p_cur_1, p_img_0, p_img_1, Dk, ...
    a_max, eB, diverge, varargin)

% Search circle algorithm for finding next point of the manifold
%
% fhandle  - function (dynamical system) handle
% iters    - number of iterates
% p_cur_0  - previous computed point of the manifold (p_{k-1})
% p_cur_1  - last computed point of the manifold (p_k)
% p_img_0  - image of previous computed point (p_{i-1})
% p_img_1  - image of the last computed point (p_i)
% Dk       - current step size (circle radius)
% a_max    - maximal angle (curvature)
% eB       - bisection error
% varargin - cell array of the function parameters
%
% return values:
% p_next - next manifold point 
% f_next - image of the next manifold point
% stat   - exit status (0: normal exit, 1: angle search
%          range was enlarged, 2: after max iterations
%          bisection accuracy not reached, -1: start or end
%          angle not found, bisection could not be
%          performed)
%


%diverge = 100000;

%cprintf(get_colour('warn'), 'search circle: %d', nargin);


% maximal number of bisection steps
load('data/default.mat', 'bstep_max');

% angle increment for finding correct p_start, p_end
load('data/default.mat', 'da');



stat = 0;


th_next = find_angle(p_cur_0, p_cur_1);

th_start = th_next - a_max;
p_start = get_cartesian(p_cur_1, Dk, th_start);
f_start = iterate(fhandle, p_start, iters, diverge, varargin{:});
while (is_clockwise(f_start, p_img_0, p_img_1) ~= -1)
    th_start = th_start - da;
    if (th_next - a_max - th_start > 2*pi)
        % start angle not found
        stat = -1;
        p_next = p_start;
        f_next = f_start;
        return;
    end
    
    stat = 1; % alpha range was enlarged
    p_start = get_cartesian(p_cur_1, Dk, th_start);
    f_start = iterate(fhandle, p_start, iters, diverge, varargin{:});
end

th_end = th_next + a_max;
p_end = get_cartesian(p_cur_1, Dk, th_end);
f_end = iterate(fhandle, p_end, iters, diverge, varargin{:});
while (is_clockwise(f_end, p_img_0, p_img_1) ~= 1)
    th_end = th_end + da;
    if (th_end - th_next - a_max > 2*pi)
        % end angle not found
        stat = -1;
        p_next = p_end;
        f_next = f_end;
        return;
    end
    
    stat = 1; % alpha range was enlarged
    p_end = get_cartesian(p_cur_1, Dk, th_end);
    f_end = iterate(fhandle, p_end, iters, diverge, varargin{:});
end

% f_start and f_end are on the right sides of [p_img_0, p_img_1]
% starting bisection
if (stat == 1)
    % alpha range changed
    th_next = (th_start + th_end)/2;
end

p_next = get_cartesian(p_cur_1, Dk, th_next);
f_next = iterate(fhandle, p_next, iters, diverge, varargin{:});
side = is_clockwise(f_next, p_img_0, p_img_1);

bstep = 1;

while (bstep <= bstep_max)
    % bisection accuracy is still not reached
    if (side == -1)
        % the same side as f_start
        % moving start point
        th_start = th_next;
    else
        % the same side as f_end
        % moving end point
        th_end = th_next;
    end
    
    th_next = (th_start + th_end)/2;
    p_next = get_cartesian(p_cur_1, Dk, th_next);
    f_next = iterate(fhandle, p_next, iters, diverge, varargin{:});
    side = is_clockwise(f_next, p_img_0, p_img_1);
    
    if (dist_point_segm(f_next, p_img_0, p_img_1) < eB)
        % desired accuracy reached
        break;
    end
    
    bstep = bstep + 1;
end % while (bstep <= bstep_max)

if (bstep > bstep_max)
    % loop ended due to max iterations number was exceeded
    % p_next is the last computed value, may be inacuurate
    stat = 2;
end


